/*---------------------------------------------------------------------------*
 |          Copyright (c) 1997-2010 PDFlib GmbH. All rights reserved.        |
 +---------------------------------------------------------------------------+
 |    This software may not be copied or distributed except as expressly     |
 |    authorized by PDFlib GmbH's general license agreement or a custom      |
 |    license agreement signed by PDFlib GmbH.                               |
 |    For more information about licensing please refer to www.pdflib.com.   |
 *---------------------------------------------------------------------------*/

/* $Id: ploplibdl.h,v 1.1 2010/09/07 13:05:05 stm Exp $
 *
 * Function prototypes for dynamically loading the PLOP DLL at runtime
 */

#ifdef __cplusplus
typedef struct PLOP_s PLOP_cpp;
#define PLOP PLOP_cpp
#define PLOP_CONTEXT_DEFINED
#endif

#include "ploplib.h"

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Notes for using the PLOP DLL loading mechanism:
 *
 * - PLOP_TRY_DL()/PLOP_CATCH_DL() must be used instead of the standard
 *   exception handling macros.
 * - PLOP_new_dl() must be used instead of PLOP_new()/PLOP_new2().
 * - PLOP_delete_dl() must be used instead of PLOP_delete().
 * - PLOP_get_opaque() must not be used.
 */

/* Load the PLOP DLL, and fetch pointers to all exported functions. */
PDFLIB_API const PLOP_api * PDFLIB_CALL
PLOP_new_dl(PLOP **p_plop);

/* Delete the PLOP object and unload the previously loaded PLOP DLL */
PDFLIB_API void PDFLIB_CALL
PLOP_delete_dl(const PLOP_api *PDFlib, PLOP *plop);

#define PLOP_TRY_DL(PLOPAPI, PLOPOBJ)	\
    if (PLOPOBJ) { if (setjmp(PLOPAPI->plop_jbuf(PLOPOBJ)->jbuf) == 0)

/* Inform the exception machinery that a PLOP_TRY_DL() will be left without
   entering the corresponding PLOP_CATCH_DL() clause. */
#define PLOP_EXIT_TRY_DL(PLOPAPI, PLOPOBJ) PLOPAPI->plop_exit_try(PLOPOBJ)

/* Catch an exception; must always be paired with PLOP_TRY_DL(). */
#define PLOP_CATCH_DL(PLOPAPI, PLOPOBJ) } if (PLOPAPI->plop_catch(PLOPOBJ))

/* Re-throw an exception to another handler. */
#define PLOP_RETHROW_DL(PLOPAPI, PLOPOBJ) PLOPAPI->plop_rethrow(PLOPOBJ)

#ifdef __cplusplus
#undef PLOP
}	/* extern "C" */
#endif
