/* $Id: sign.c,v 1.23 2008/11/24 23:31:54 tm Exp $
 *
 * PDFlib PLOP: PDF Linearization, Optimization, Protection
 * digital signature sample
 *
 */

#include <stdio.h>

#include "ploplib.h"

int
main(int argc, char **argv)
{
    PLOP *plop;

    /* This is where input files live. Adjust as necessary. */
    char *searchpath = "../data";

    if (argc != 3)
    {
	printf("usage: sign <infile> <outfile>\n");
	return 2;
    }

    if ((plop = PLOP_new()) == (PLOP *) 0)
    {
	printf("Couldn't create PLOP object (out of memory)!\n");
	return(2);
    }

    PLOP_TRY(plop)
    {
	int doc;
	char optlist[1024];

	static const char sign_opts[] = "sign={"
	    "appearance={fieldname=Signature1} "
#if defined(USE_MSCAPI)
	    "engine=mscapi "
	    "digitalid={certstore={store=My subject={DEMO PLOP user 2048}}} "
#elif defined(USE_PKCS11)
	    "engine=pkcs#11 "
	    "digitalid={filename=cryptoki.dll} "
#else
	    "engine=builtin "
	    "digitalid={filename=demo2048.p12} "
#endif
	    "password={demo} "
	    "reason={I agree to this document} "
	    "contactinfo={tel (123) 456 789} "
	    "location=Munich "
	    "}";

	sprintf(optlist, "searchpath={%s}", searchpath);
	PLOP_set_option(plop, optlist);

	if ((doc = PLOP_open_document(plop, argv[1], 0, "")) == -1)
	{
	    printf("Error: %s\n", PLOP_get_errmsg(plop));
	    PLOP_delete(plop);
	    return(2);
	}

	if (PLOP_create_file(plop, argv[2], 0, sign_opts) == -1)
	{
	    printf("Error: %s\n", PLOP_get_errmsg(plop));
	    PLOP_delete(plop);
	    return(2);
	}

	PLOP_close_document(plop, doc);
    }

    PLOP_CATCH(plop)
    {
	printf("PLOP exception occurred in sign:\n");
	printf("[%d] %s: %s\n", PLOP_get_errnum(plop),
		PLOP_get_apiname(plop), PLOP_get_errmsg(plop));
	PLOP_delete(plop);
	return(2);
    }

    PLOP_delete(plop);

    return(0);
}
