/* $Id: insertxmp.cpp,v 1.5 2010/11/05 13:06:20 stm Exp $
 *
 * PDFlib PLOP: PDF Linearization, Optimization, Protection
 * mini sample for inserting XMP metadata in PDF documents
 *
 */

#include <iostream>
#include <string>

#include "plop.hpp"

using namespace std;
using namespace pdflib;

int
main(void)
{
    PLOP plop;

    /* Directory where files will be searched */
    const wstring searchpath(L"../data");

    /* input document */
    const wstring in_filename(L"PLOP-datasheet.pdf");

    /* output document */
    const wstring out_filename(L"PLOP-datasheet-xmp.pdf");

    /* Option list for the generated output document.
     * Some alternatives for the option list using the supplied XMP samples:
     *
     * Insert custom metadata for the GWG (Ghent Workgroup)
     * Ad Ticket scheme (see http://www.gwg.org/Jobtickets.phtml):
     * L"metadata={filename=gwg_ad_ticket.xmp validate=xmp2004}"
     *
     * Insert custom metadata based on Acme's enterprise schema:
     * L"metadata={filename=acme.xmp}"
     *
     * Insert a custom L"engineering" schema for scanned documents:
     * This works best with a PDF/A-1 input document (e.g. mini_pdfa1.pdf),
     * and requires XMP input which conforms to PDF/A-1:
     * L"metadata={filename=engineering.xmp}"
     *
     * Insert a PDF/A extension schema after validating the XMP for PDF/A-1.
     * This works best with a PDF/A-1 input document (e.g. mini_pdfa1.pdf),
     * and requires XMP input which conforms to PDF/A-1:
     * L"metadata={filename=machine_pdfa1.xmp validate=pdfa1}"
     *
     */
    const wstring out_optlist = L"metadata={filename=gwg_ad_ticket.xmp}";


    try {
	int doc;
	wstring optlist;

	optlist = L"searchpath={" + searchpath + L"}";
	plop.set_option(optlist);

	/* open input file */
	if ((doc = plop.open_document(in_filename, L"")) == -1)
	{
	    wcerr << L"Error: " << plop.get_errmsg() << endl;
	    return(2);
	}

	/* create the output file and add XMP metadata */
	if (plop.create_file(out_filename, out_optlist) == -1)
	{
	    wcerr << L"Error: " << plop.get_errmsg() << endl;
	    return(2);
	}

	/* close input and output files */
	plop.close_document(doc);
    }

    catch (PLOP::Exception &ex) {
	wcerr << L"PLOP exception occurred in insertxmp:" << endl;
	wcerr << L"[" << ex.get_errnum() << L"] " << ex.get_apiname()
		    << L": " << ex.get_errmsg() << endl;
	return(2);
    }

    return(0);
}
