/*---------------------------------------------------------------------------*
 |          Copyright (c) 2005-2010 PDFlib GmbH. All rights reserved.        |
 +---------------------------------------------------------------------------+
 |    This software may not be copied or distributed except as expressly     |
 |    authorized by PDFlib GmbH's general license agreement or a custom      |
 |    license agreement signed by PDFlib GmbH.                               |
 |    For more information about licensing please refer to www.pdflib.com.   |
 *---------------------------------------------------------------------------*/

// $Id: plop.hpp,v 1.21 2011/02/24 13:19:12 rjs Exp $
//
// C++ wrapper for PLOP
//
//

#ifndef PLOPLIB_HPP
#define PLOPLIB_HPP

#include <string>
#include <sstream>
#include <iostream>
#include <stdexcept>

// We use PLOP as a C++ class name, therefore hide the actual C struct
// name for PLOP usage with C++.

typedef struct PLOP_s PLOP_cpp;
#if _MSC_VER >= 1310    /* VS .NET 2003 and later */
/* ploplib.h declares some functions as deprecated, but we don't want to see
 * these warnings here */
#pragma warning(disable: 4995)
#endif

#define PLOP PLOP_cpp
#include "ploplib.h"
#undef PLOP

#if _MSC_VER >= 1310    /* VS .NET 2003 and later */
#pragma warning(enable: 4995)
#endif
/*
 * PLOPCPP_USE_PDFLIB_NAMESPACE
 *
 * If set to a non-zero value, wrap all declarations in the namespace "PDFlib".
 * Otherwise no namespace declaration is created. For backward compatibility
 * with the PLOP 4 C++ wrapper, define this as 0.
 */
#ifndef PLOPCPP_USE_PDFLIB_NAMESPACE
#define PLOPCPP_USE_PDFLIB_NAMESPACE 1
#endif

/*
 * PLOPCPP_PDFLIB_WSTRING
 *
 * The plop.hpp header declares one instantiation of the basic_PLOP
 * template as type "PLOP". By default this PLOP type is std::wstring-based.
 * For backward compatibility with the PLOP 4 std::string-based C++ wrapper,
 * define this as 0.
 */
#ifndef PLOPCPP_PLOP_WSTRING
#define PLOPCPP_PLOP_WSTRING 1
#endif

/*
 * PLOPCPP_DL
 *
 * The plop.hpp header can be used for static linking against the PLOP
 * library, or it can be configured for loading the PLOP DLL dynamically at
 * runtime.
 *
 * The default is to compile for static linking against the PLOP library. For
 * dynamic loading, define PLOPCPP_DL as 1. In that case the resulting
 * program must not be linked against the PLOP library. Instead the ploplibdl.c
 * module from the "bind/c" directory must be compiled and linked to the
 * application.
 */
#ifndef PLOPCPP_DL
#define PLOPCPP_DL 0
#endif

#if PLOPCPP_DL
#include "ploplibdl.h"
#endif


#if defined(_MSC_VER) && defined(_MANAGED)
/*
 * Dummy declaration to prevent linker warning for .NET. If it doesn't see
 * a declaration of the structure it will complain. The structure is never
 * used in the .NET wrapper, so it is safe to declare it as an empty structure
 * here.
 */
struct PLOP_s {};
#pragma unmanaged
#endif

#if PLOPCPP_USE_PDFLIB_NAMESPACE
namespace pdflib {
#endif

template<class pstring, class conv> class basic_PLOP;

/**
 * The "do-nothing" converter that has the effect that the basic_PLOP class
 * behaves in the same way as the PLOP 4 wrapper.
 */
template<class pstring>
class PLOPNoOpConverter
{
public:
    static bool do_conversion()
    {
        return false;
    }

    static void convert_to_pdf_bytes(
        const basic_PLOP<pstring, PLOPNoOpConverter<pstring> >& p,
        const pstring& in, std::string& out)
    {
        throw std::logic_error(
                "PLOPNoOpConverter::convert_to_pdf_bytes: internal error: "
                "converter called although do_conversion() returns false");
    }

    static void convert_to_pdf_utf8(
            const basic_PLOP<pstring, PLOPNoOpConverter<pstring> >& p,
            const pstring&, std::string&)
    {
        throw std::logic_error(
                "PLOPNoOpConverter::convert_to_pdf_utf8: internal error: "
                "converter called although do_conversion() returns false");
    }

    static void convert_to_pdf_utf16(
            const basic_PLOP<pstring, PLOPNoOpConverter<pstring> >& p,
            const pstring&, std::string&)
    {
        throw std::logic_error(
                "PLOPNoOpConverter::convert_to_pdf_utf16: internal error: "
                "converter called although do_conversion() returns false");
    }

    static void convert_to_pstring(
            const basic_PLOP<pstring, PLOPNoOpConverter<pstring> >& p,
            const char *, pstring&)
    {
        throw std::logic_error(
                "PLOPNoOpConverter::convert_to_pstring: internal error: "
                "converter called although do_conversion() returns false");
    }
};

#if defined(_MSC_VER) && defined(_MANAGED)
/*
 * Switching to C++ try/catch for exception handling allows .NET compilation
 * of the C++ wrapper with "/clr" and also with "/clr:pure".
 */
namespace
{

template <class pstring, class conv>
void
errorhandler(PLOP_cpp *plop, int errortype, const char* msg)
{
    /*
     * The PLOP API returns the error message and the API name in the
     * expected encoding.
     */
    pstring message(reinterpret_cast<const typename pstring::value_type *>
                                        (PLOP_get_errmsg(plop)));
    pstring apiname(reinterpret_cast<const typename pstring::value_type *>
                                        (PLOP_get_apiname(plop)));

    throw typename basic_PLOP<pstring, conv>::Exception(
            message, PLOP_get_errnum(plop), apiname, PLOP_get_opaque(plop));
}

}

#define PLOPCPP_TRY try {

#define PLOPCPP_CATCH } catch (Exception &) {\
  throw;\
}

#else /* defined(_MSC_VER) && defined(_MANAGED) */

#if PLOPCPP_DL

#define PLOPCPP_TRY      PLOP_TRY_DL(m_PLOP_api, p) {
#define PLOPCPP_CATCH  \
} PLOP_CATCH_DL(m_PLOP_api, p) {\
    const pstring message(reinterpret_cast<const typename pstring::value_type *> \
        (m_PLOP_api->PLOP_get_errmsg(p))); \
    const pstring apiname(reinterpret_cast<const typename pstring::value_type *> \
        (m_PLOP_api->PLOP_get_apiname(p))); \
    throw Exception(message, m_PLOP_api->PLOP_get_errnum(p), apiname, \
                        m_PLOP_api->PLOP_get_opaque(p)); \
}

#else

#define PLOPCPP_TRY      PLOP_TRY(p) {
#define PLOPCPP_CATCH  \
} PLOP_CATCH(p) { \
    const pstring message(reinterpret_cast<const typename pstring::value_type *> \
        (m_PLOP_api->PLOP_get_errmsg(p))); \
    const pstring apiname(reinterpret_cast<const typename pstring::value_type *> \
        (m_PLOP_api->PLOP_get_apiname(p))); \
    throw Exception(message, m_PLOP_api->PLOP_get_errnum(p), apiname, \
                        m_PLOP_api->PLOP_get_opaque(p)); \
}

#endif

#endif /* !(defined(_MSC_VER) && defined(_MANAGED)) */

template<class pstring, class conv>
class basic_PLOP
{
    friend class PLOPNoOpConverter<pstring>;

public:
    class Exception
    {
    public:
        Exception(const pstring& errmsg, int errnum, const pstring& apiname,
                    void *opaque) :
            m_errmsg(errmsg),
              m_errnum(errnum),
              m_apiname(apiname),
              m_opaque(opaque)
        {
        }
        pstring get_errmsg() const { return m_errmsg; }
        int get_errnum() const { return m_errnum; }
        pstring get_apiname() const { return m_apiname; }
        const void *get_opaque() const { return m_opaque; }
    private:
        pstring m_errmsg;
        int m_errnum;
        pstring m_apiname;
        void * m_opaque;
    };

#if defined(_MSC_VER) && defined(_MANAGED)
    basic_PLOP(void *opaque = NULL) :
        m_PLOP_api(::PLOP_get_api())
    {
        check_api(opaque);

        p = m_PLOP_api->PLOP_new2(errorhandler<pstring, conv>, opaque);
        if (!p)
        {
            throw std::bad_alloc();
        }

        set_cpp_binding_options();
    }

    ~basic_PLOP()
    {
        m_PLOP_api->PLOP_delete(p);
    }
#else

#if PLOPCPP_DL
    class dl_load_error: public std::runtime_error
    {
    public:
        explicit dl_load_error() :
            std::runtime_error("Couldn't load PLOP DLL")
        {
        }
    };

    /*
     * The dynamic loading variant of the constructor accepts the "opaque"
     * parameter, but requires it to be NULL, as the opaque pointer is used
     * internally by PLOP_new_dl().
     */
    basic_PLOP(void *opaque = NULL)
    {
        if (opaque)
        {
            throw std::invalid_argument(
                    "In the dynamic loading variant of the PLOP C++ binding "
                    "the 'opaque' parameter must be NULL");
        }

        m_PLOP_api = PLOP_new_dl(&p);

        if (!m_PLOP_api)
        {
            throw dl_load_error();
        }

        check_api(NULL);

        set_cpp_binding_options();
    }

    ~basic_PLOP()
    {
        PLOP_delete_dl(m_PLOP_api, p);
    }
#else
    basic_PLOP(void *opaque = NULL) :
        m_PLOP_api(::PLOP_get_api())
    {
        check_api(opaque);

        p = m_PLOP_api->PLOP_new2(NULL, opaque);
        if (!p)
        {
            throw std::bad_alloc();
        }

        set_cpp_binding_options();
    }

    ~basic_PLOP()
    {
        m_PLOP_api->PLOP_delete(p);
    }
#endif

#endif

    void *
    get_opaque()
    {
        void *retval = NULL;

        PLOPCPP_TRY
            retval = PLOP_get_opaque(p);
        PLOPCPP_CATCH;

        return retval;
    }

#include "cpp_wrapped.h"

    /* Deprecated, use PLOP_convert_to_unicode(). */
    std::string utf16_to_utf8(const std::string utf16string) const
    {
        const char *retval = NULL;

        PLOPCPP_TRY
            retval = m_PLOP_api->PLOP_utf16_to_utf8(p, utf16string.c_str(),
                    static_cast<int>(utf16string.length()), NULL);
        PLOPCPP_CATCH;

        return retval ? retval : "";
    }

    /* Deprecated, use PLOP_convert_to_unicode(). */
    std::string utf8_to_utf16(const std::string& utf8string,
                const std::string& ordering) const
    {
        int size;
        const char *buf;
        std::string retval;

        PLOPCPP_TRY
            buf = m_PLOP_api->PLOP_utf8_to_utf16(p, utf8string.c_str(),
						    ordering.c_str(), &size);
            if (buf)
                retval.assign(buf, static_cast<size_t>(size));
        PLOPCPP_CATCH;

        return retval;
    }

    
    /* Convert an arbitrary encoded string to a Unicode string in
     * several formats. */
    std::string
    convert_to_unicode(const pstring& inputformat,
            const std::string& inputstring, const pstring& optlist) const
    {
	const char *retval = NULL;
	std::string retstring;

	std::string inputformat_param;
	const char *p_inputformat_param;
	param_to_utf8(inputformat, inputformat_param, p_inputformat_param);

	std::string optlist_param;
	const char *p_optlist_param;
	param_to_utf8(optlist, optlist_param, p_optlist_param);

	int outputlen;

	PLOPCPP_TRY {
	    retval = m_PLOP_api->PLOP_convert_to_unicode(p,
		p_inputformat_param,
		static_cast<const char *>(inputstring.c_str()),
		inputstring.length(), &outputlen, p_optlist_param);
	}
	PLOPCPP_CATCH;

	if (retval)
	    retstring.assign(retval, static_cast<size_t>(outputlen));

	return retstring;
    }


    /* Open a PDF document (which may be protected) via a user-supplied
     * function. */
    int
    open_document_callback(void *opaque, size_t filesize,
            size_t (*readproc)(void *opaque, void *buffer, size_t size),
            int (*seekproc)(void *opaque, long offset), const pstring& optlist)
    {
        int retval = -1;

        std::string optlist_param;
        const char *p_optlist_param;
        param_to_utf8(optlist, optlist_param, p_optlist_param);

        PLOPCPP_TRY
            retval = m_PLOP_api->PLOP_open_document_callback(p,
		opaque, filesize, readproc, seekproc, p_optlist_param);
        PLOPCPP_CATCH;

        return retval;
    }


    

protected:
    const PLOP_api *m_PLOP_api;
    PLOP_cpp *p;

private:
    void set_cpp_binding_options(void)
    {
        PLOPCPP_TRY
        {
            m_PLOP_api->PLOP_set_option(p, "objorient");
            if (conv::do_conversion())
            {
                m_PLOP_api->PLOP_set_option(p, "binding={C++ conv}");
            }
            else
            {
                switch (sizeof(typename pstring::value_type))
                {
                case sizeof(char):
                    m_PLOP_api->PLOP_set_option(p,
                        "binding={C++ legacy} apitextformat=utf8");
                    break;

                case utf16_wchar_t_size:
                    m_PLOP_api->PLOP_set_option(p,
                        "binding={C++} unicaplang=true apitextformat=utf16");
                    break;

                case utf32_wchar_t_size:
                    m_PLOP_api->PLOP_set_option(p,
                        "binding={C++} unicaplang=true apitextformat=utf32");
                    break;

                default:
                    bad_wchar_size("basic_PLOP<pstring, conv>::basic_PLOP");
                }
            }
        }
        PLOPCPP_CATCH;
    }

    void check_api(void *opaque)
    {
        if (m_PLOP_api->sizeof_PLOP_api != sizeof(PLOP_api) ||
                m_PLOP_api->major != PLOP_MAJORVERSION ||
                m_PLOP_api->minor != PLOP_MINORVERSION)
        {
            pstring message;
            pstring apiname; /* stays empty */

            switch (sizeof(typename pstring::value_type))
            {
            case sizeof(char):
                apiretval_to_pstring("loaded wrong version of PLOP library", message);
                break;

            case utf16_wchar_t_size:
            case utf32_wchar_t_size:
                apiretval_to_pstring(reinterpret_cast<const char *>(L"loaded wrong version of PLOP library"), message);
                break;

            default:
                bad_wchar_size("basic_PLOP<pstring, conv>::check_api");
            }

            throw Exception(message, -1, apiname, opaque);
        }
    }

    enum
    {
        utf16_wchar_t_size = 2,
        utf32_wchar_t_size = 4
    };

    void bad_wchar_size(const char *apiname) const
    {
        std::ostringstream exception_text;
        exception_text << apiname << ": unsupported wchar_t size: "
                        << sizeof(typename pstring::value_type);

        throw std::logic_error(exception_text.str());
    }

    void param_to_utf8(const pstring& param, std::string& PLOP_param,
                        const char *& PLOP_ptr) const
    {
        if (conv::do_conversion())
        {
            conv::convert_to_pdf_utf8(*this, param, PLOP_param);
            PLOP_ptr = PLOP_param.c_str();
        }
        else
        {
            const char *s = reinterpret_cast<const char *>(param.c_str());

            switch (sizeof(typename pstring::value_type))
            {
            case sizeof(char):
                /*
                 * Legacy case: Pass through user-supplied string.
                 */
                PLOP_ptr = s;
                break;

            case utf16_wchar_t_size:
                PLOP_ptr =
                    m_PLOP_api->PLOP_utf16_to_utf8(p, s,
                        static_cast<int>(param.length() * sizeof(wchar_t)), 0);
                break;

            case utf32_wchar_t_size:
                PLOP_ptr =
                    m_PLOP_api->PLOP_utf32_to_utf8(p, s,
                        static_cast<int>(param.length() * sizeof(wchar_t)), 0);
                break;

            default:
                bad_wchar_size("basic_PLOP<pstring, conv>::param_to_utf8");
            }
        }
    }

    void param_to_0utf16(const pstring& param, std::string& PLOP_param,
                        const char *& PLOP_ptr, int& len) const
    {
        if (conv::do_conversion())
        {
            conv::convert_to_pdf_utf16(*this, param, PLOP_param);
            PLOP_ptr = PLOP_param.c_str();
            len = static_cast<int>(PLOP_param.length());
        }
        else
        {
            const char *s = reinterpret_cast<const char *>(param.c_str());

            switch (sizeof(typename pstring::value_type))
            {
            case sizeof(char):
                /*
                 * Legacy case: Pass through user-supplied string with length 0,
                 * string must not contain 0 bytes.
                 */
                PLOP_ptr = s;
                len = 0;
                break;

            case utf16_wchar_t_size:
                /*
                 * UTF-16 can also be passed through directly
                 */
                PLOP_ptr = s;
                len = static_cast<int>(param.length() * utf16_wchar_t_size);
                break;

            case utf32_wchar_t_size:
                PLOP_ptr =
                    m_PLOP_api->PLOP_utf32_to_utf16(p, s,
                        static_cast<int>(param.length() * utf32_wchar_t_size),
                        "", &len);
                break;

            default:
                bad_wchar_size("basic_PLOP<pstring, conv>::param_to_utf16");
            }
        }
    }

    void param_to_utf16(const pstring& param, std::string& PLOP_param,
                        const char *& PLOP_ptr, int& len) const
    {
        if (conv::do_conversion())
        {
            conv::convert_to_pdf_utf16(*this, param, PLOP_param);
            PLOP_ptr = PLOP_param.c_str();
            len = static_cast<int>(PLOP_param.length());
        }
        else
        {
            const char *s = reinterpret_cast<const char *>(param.c_str());

            switch (sizeof(typename pstring::value_type))
            {
            case sizeof(char):
                /*
                 * Legacy case: Pass through user-supplied string including
                 * explicit length, string may contain 0 bytes.
                 */
                PLOP_ptr = s;
                len = static_cast<int>(param.length());
                break;

            case utf16_wchar_t_size:
                /*
                 * UTF-16 can also be passed through directly
                 */
                PLOP_ptr = s;
                len = static_cast<int>(param.length() * utf16_wchar_t_size);
                break;

            case utf32_wchar_t_size:
                PLOP_ptr =
                    m_PLOP_api->PLOP_utf32_to_utf16(p, s,
                        static_cast<int>(param.length() * utf32_wchar_t_size),
                        "", &len);
                break;

            default:
                bad_wchar_size("basic_PLOP<pstring, conv>::param_to_utf16");
            }
        }
    }

    void param_to_bytes(const pstring& param, std::string& PLOP_param,
                        const char *& PLOP_ptr) const
    {
        if (conv::do_conversion())
        {
            conv::convert_to_pdf_bytes(*this, param, PLOP_param);
            PLOP_ptr = PLOP_param.c_str();
        }
        else
        {
            const size_t size = sizeof(typename pstring::value_type);
            const char *s = reinterpret_cast<const char *>(param.c_str());

            switch (size)
            {
            case sizeof(char):
                PLOP_ptr = s;
                break;

            case utf16_wchar_t_size:
            case utf32_wchar_t_size:
                {
                    int highchar;

                    const char *deflated =
                            m_PLOP_api->PLOP_deflate_unicode(p, s,
                                    static_cast<int>(param.length() * size),
                                    size, &highchar);

                    if (!deflated)
                    {
                        std::ostringstream exception_text;

                        exception_text
                            << "basic_PLOP::param_to_bytes: high "
                                "Unicode character '0x"
                            << std::hex << highchar
                            << "' is not supported in this character string";

                        throw std::runtime_error(exception_text.str().c_str());
                    }

                    PLOP_ptr = deflated;
                }
                break;

            default:
                bad_wchar_size("basic_PLOP<pstring, conv>::param_to_bytes");
            }
        }
    }

    void apiretval_to_pstring(const char * const PLOP_retval,
                        pstring& cpp_retval) const
    {
        if (conv::do_conversion())
        {
            if (PLOP_retval)
            {
                conv::convert_to_pstring(*this, PLOP_retval, cpp_retval);
            }
            else
            {
                cpp_retval.clear();
            }
        }
        else
        {
            if (PLOP_retval)
            {
                cpp_retval.assign(reinterpret_cast
                                    <const typename pstring::value_type *>
                                        (PLOP_retval));
            }
            else
            {
                cpp_retval.clear();
            }
        }
    }

    /**
     * Separate routine for converting output strings to pstrings. This is used
     * for the return value of get_text(), where the length is needed because
     * the wchar_t strings are not 0-terminated.
     */
    void outputstring_to_pstring(const char * const PLOP_retval,
                        pstring& cpp_retval, const int length) const
    {
        if (conv::do_conversion())
        {
            if (PLOP_retval)
            {
                conv::convert_to_pstring(*this, PLOP_retval, cpp_retval);
            }
            else
            {
                cpp_retval.clear();
            }
        }
        else
        {
            if (PLOP_retval)
            {
                cpp_retval.assign(reinterpret_cast
                                    <const typename pstring::value_type *>
                                        (PLOP_retval),
                                    length);
            }
            else
            {
                cpp_retval.clear();
            }
        }
    }

    /*
     * Prevent copy construction and assignment because of private PLOP_cpp
     * pointer.
     */
    basic_PLOP(const basic_PLOP&);
    basic_PLOP& operator=(const basic_PLOP&);
};

#if PLOPCPP_PLOP_WSTRING
typedef basic_PLOP<std::wstring, PLOPNoOpConverter<std::wstring> > PLOP;
#else
typedef basic_PLOP<std::string, PLOPNoOpConverter<std::string> > PLOP;
#endif

#if PLOPCPP_USE_PDFLIB_NAMESPACE
} // end of PDFlib namespace
#endif

#endif	// PLOPLIB_HPP
