/* $Id: sign.cpp,v 1.17 2011/02/23 11:45:53 stm Exp $
 *
 * PDFlib PLOP: PDF Linearization, Optimization, Protection
 * digital signature sample
 *
 */

#include <iostream>
#include <string>

#include "plop.hpp"

using namespace std;
using namespace pdflib;

namespace
{
wstring get_wstring(const PLOP& plop, const string& utf8_string);
}

int
main(int argc, char **argv)
{
    PLOP plop;

    /* This is where input files live. Adjust as necessary. */
    const wstring searchpath(L"../data");

    if (argc != 3)
    {
	wcerr << L"usage: sign <infile> <outfile>" << endl;
	return 2;
    }

    try
    {
	const wstring optlist = L"searchpath={" + searchpath + L"}";

	const wstring sign_opts(L"sign={"
	    L"appearance={fieldname=Signature1} "
#if defined(USE_MSCAPI)
	    L"engine=mscapi "
	    L"digitalid={certstore={store=My subject={DEMO PLOP user 2048}}} "
#elif defined(USE_PKCS11)
	    L"engine=pkcs#11 "
	    L"digitalid={filename=cryptoki.dll} "
#else
	    L"engine=builtin "
	    L"digitalid={filename=demo2048.p12} "
#endif
	    L"password={demo} "
	    L"reason={I agree to this document} "
	    L"contactinfo={tel (123) 456 789} "
	    L"location=Munich "
	    L"}");

	plop.set_option(optlist);

        /*
         * Caution: For simplicity we assume that the program arguments are
         * encoded as UTF-8, which might not be true in all cases!
         */
        const wstring doc_name(get_wstring(plop, string(argv[1])));
        const int doc = plop.open_document(doc_name, L"");
	if (doc == -1)
	{
	    wcerr << L"Error: " << plop.get_apiname()
                        << L": " << plop.get_errmsg() << endl;
	    return 2;
	}

        const wstring out_name(get_wstring(plop, string(argv[2])));
	if (plop.create_file(out_name, sign_opts) == -1)
	{
	    wcerr << L"Error: " << plop.get_apiname()
                        << L": " << plop.get_errmsg() << endl;
	    return 2;
	}

	plop.close_document(doc);
    }
    catch (PLOP::Exception &ex)
    {
	wcerr << L"PLOP exception occurred in encrypt:" << endl;
	wcerr << L"[" << ex.get_errnum() << L"] " << ex.get_apiname()
		<< L": " << ex.get_errmsg() << endl;
	return 2;
    }

    return 0;
}

namespace
{

/*
 * Get a wstring for the given string, assuming the string is encoded in
 * UTF-8.
 */
wstring get_wstring(const PLOP& plop, const string& utf8_string)
{
    const size_t size = sizeof(wstring::value_type);
    string wide_string;

    switch (size)
    {
    case 2:
        wide_string = plop.convert_to_unicode(L"utf8",
                            utf8_string, L"outputformat=utf16");
        break;

    case 4:
        wide_string = plop.convert_to_unicode(L"utf8",
                            utf8_string, L"outputformat=utf32");
        break;

    default:
        throw std::logic_error("Unsupported wchar_t size");
    }

    return wstring(reinterpret_cast<const wchar_t *>(wide_string.data()),
            wide_string.length() / size);
}

} // end anonymous namespace

