/* $Id: hellosign.java,v 1.12 2011/02/23 18:34:58 rjs Exp $
 *
 * PDFlib PLOP: PDF Linearization, Optimization, Protection
 * hellosign sample: generate a document in memory with PDFlib, and sign it
 * with PLOP
 */

import java.io.*;
import com.pdflib.PLOPException;
import com.pdflib.plop;
import com.pdflib.pdflib;

public class hellosign
{
    public static void main (String argv[])
    {
	pdflib pdf = null;
	plop plop = null;

	try {
	    int font;
	    byte[] contents;

	    final String out_filename = "hellosign.pdf";
	    String optlist;
	    
	    String searchpath = "../data";
	    
	    /*
	     * alternatives for specifying the certificate
	     *
	     * Windows Certificate Store (Windows only):
	     *    "engine=mscapi digitalid={certstore={store=My subject={DEMO PLOP user 2048}}} "
	     * PFX certificate from file in search path:
	     *    "digitalid={filename=demo2048.pfx} "
	     * PKCS#12 certificate from file in search path:
	     *     "digitalid={filename=demo2048.p12} "
	     */
	    String sign_opts = "sign={"  +
		"appearance={fieldname=Signature1 } " +
		"digitalid={filename=demo2048.p12} " +
		"password={demo} " +
		"reason={I agree to this document} " +
		"contactinfo={tel (123) 456 789} " +
		"location=Munich " +
            "}";
            
	    int doc;

	    /* ----------- Step 1: generate the document in memory ---------- */

	    pdf = new pdflib();

	    pdf.begin_document("", ""); 

	    pdf.set_info("Creator", "hellosign.java");
	    pdf.set_info("Author", "Rainer Ploeckl");
	    pdf.set_info("Title", "Hello world/signed (Java)");

	    font = pdf.load_font("Helvetica-Bold", "winansi", "");

	    pdf.begin_page_ext(595, 842, "");
	    pdf.setfont(font, 24);
	    pdf.set_text_pos(50, 700);
	    pdf.show("Hello world/signed!");
	    pdf.continue_text("(says Java)");
	    pdf.end_page_ext("");

	    pdf.end_document("");
	    contents = pdf.get_buffer();

	    /* --------------- Step 2: sign the document -------------- */

	    plop = new plop();

	    optlist = "searchpath {" + searchpath + "} ";
	    plop.set_option(optlist);

	    /* Create a virtual PVF file from the data in memory */
            plop.create_pvf("/pvf/hello.pdf", contents, "");

            /* open unsigned input document from PVF file */
            if ((doc = plop.open_document("/pvf/hello.pdf", "")) == -1) {
		throw new Exception("Error: " + plop.get_errmsg());
	    }


	    /* sign the output file */
	    if (plop.create_file(out_filename, sign_opts) == -1) {
		throw new Exception("Error: " + plop.get_errmsg());
	    }

            /* close input and output files */
            plop.close_document(doc);

	    /* delete PVF file containing input document */
	    if (plop.delete_pvf("/pvf/hello.pdf") == -1) {
		throw new Exception("Error: " + plop.get_errmsg());
	    }


        } catch (PLOPException e) {
            System.err.print("PLOP exception occurred in hellosign sample:\n");
            System.err.print("[" + e.get_errnum() + "] " + e.get_apiname() +
                            ": " + e.get_errmsg() + "\n");
        } catch (Exception e) {
            System.err.println(e.getMessage());
	} finally {
	    /* delete the PLOP object */
	    if (plop != null) plop.delete();

	    /* delete the PDFlib object */
	    if (pdf != null) {
		try { 
		    pdf.delete();
		} catch (Exception e) {
		    System.exit(2);
		}
	    }
	}
    }
}
