/* $Id:  $
 *
 * PDFlib PLOP: PDF Linearization, Optimization, Protection
 * hellosign sample: generate a document in memory with PDFlib, and sign it
 * with PLOP
 */

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;

import com.pdflib.PDFlibException;
import com.pdflib.PLOPException;
import com.pdflib.pdflib;
import com.pdflib.plop;

public class HelloSignServlet extends HttpServlet {
    private static final long serialVersionUID = 1177865317978810551L;

    public void service(HttpServletRequest request, HttpServletResponse response)
            throws ServletException {
        pdflib pdf = null;
        plop plop = null;

        try {
            final String out_filename = "";
            
            /*
             * Set up searchpath for the required data.
             */
            final ServletContext context = getServletContext();
            final String searchpath = context.getRealPath("/WEB-INF/data");

            /*
             * Catch problems that application data cannot be accessed.
             */
            if (searchpath == null) {
                throw new ServletException(
                        "Error: Unable to access application data (WAR file not unpacked?)");
            }

            /*
             * alternatives for specifying the certificate
             * 
             * Windows Certificate Store (Windows only):
             * "engine=mscapi digitalid={certstore={store=My subject={DEMO PLOP user 2048}}} "
             * PFX certificate from file in search path (Windows only):
             * "engine=mscapi digitalid={filename=demo2048.pfx} " PKCS#12
             * certificate from file in search path:
             * "digitalid={filename=demo2048.p12} "
             */
            String sign_opts = "sign={" + "appearance={fieldname=Signature1 } "
                    + "digitalid={filename=demo2048.p12} " + "password={demo} "
                    + "reason={I agree to this document} "
                    + "contactinfo={tel (123) 456 789} " + "location=Munich "
                    + "}";

            /* ----------- Step 1: generate the document in memory ---------- */

            pdf = new pdflib();

            pdf.begin_document("", "");

            pdf.set_info("Creator", "HelloSignServlet.java");
            pdf.set_info("Author", "Rainer Pl\u00f6ckl");
            pdf.set_info("Title", "Hello world/signed (Java)");

            final int font = pdf.load_font("Helvetica-Bold", "winansi", "");

            pdf.begin_page_ext(595, 842, "");
            pdf.setfont(font, 24);
            pdf.set_text_pos(50, 700);
            pdf.show("Hello world/signed!");
            pdf.continue_text("(says Java)");
            pdf.end_page_ext("");

            pdf.end_document("");
            
            final byte[] contents = pdf.get_buffer();

            /* --------------- Step 2: sign the document -------------- */

            plop = new plop();

            String optlist = "searchpath {" + searchpath + "} ";
            plop.set_option(optlist);

            /* Create a virtual PVF file from the data in memory */
            plop.create_pvf("/pvf/hello.pdf", contents, "");

            /* open unsigned input document from PVF file */
            final int doc = plop.open_document("/pvf/hello.pdf", "");

            /* sign the output file */
            if (plop.create_file(out_filename, sign_opts) == -1) {
                throw new Exception("Error: " + plop.get_errmsg());
            }

            final byte[] buf = plop.get_buffer();

            response.setContentType("application/pdf");
            response.setContentLength(buf.length);

            final ServletOutputStream out = response.getOutputStream();
            out.write(buf);
            out.close();

            /* close input and output files */
            plop.close_document(doc);

            /* delete PVF file containing input document */
            plop.delete_pvf("/pvf/hello.pdf");
        }
        catch (PLOPException e) {
            throw new ServletException("PLOP exception occurred in "
                    + getClass().getName(), e);
        }
        catch (PDFlibException e) {
            throw new ServletException("PDFlib exception occurred in "
                    + getClass().getName(), e);
        }
        catch (Exception e) {
            throw new ServletException("Exception occurred in "
                    + getClass().getName(), e);
        }
        finally {
            /* delete the PLOP object */
            if (plop != null)
                plop.delete();

            /* delete the PDFlib object */
            if (pdf != null) {
                pdf.delete();
            }
        }
    }
}
