#!/usr/bin/perl
# Id: hellosign.pl,v 1.1 2011/02/05 21:09:49 rjs Exp 
#
# PDFlib plop: PDF Linearization, Optimization, Protection
# hellosign sample: generate a document in memory with PDFlib, and sign it
# with plop
#

from sys import *
from traceback import *
from plop_py import *
from pdflib_py import *


# This is where input files live. Adjust as necessary.
searchpath = "../data"

out_filename = "hellosign.pdf"

# alternatives for specifying the certificate
#
# Windows Certificate Store (Windows only):
#    "engine=mscapi digitalid={certstore={store=My subject={DEMO PLOP user 2048}}} "
# PFX certificate from file in search path:
#    "digitalid={filename=demo2048.pfx} "
# PKCS#12 certificate from file in search path:
#    "digitalid={filename=demo2048.p12} "

sign_opts = "sign={" \
        "appearance={fieldname=Signature1 } " \
        "digitalid={filename=demo2048.p12} " \
        "password={demo} " \
        "reason={I agree to this document} " \
        "contactinfo={tel (123) 456 789} " \
        "location=Munich " \
        "}"

# create a new PDFlib object
p = PDF_new()

try:
    try:
        # open new PDF file in memory
        PDF_begin_document(p, "", "")

        PDF_set_info(p, "Creator", "hellosign.c")
        PDF_set_info(p, "Author", "Rainer Ploeckl")
        PDF_set_info(p, "Title", "Hello, world/signed (C)!")

        font = PDF_load_font(p, "Helvetica-Bold", "host", "")

        PDF_begin_page_ext(p, 595, 842, "")

        PDF_setfont(p, font, 24)
        PDF_set_text_pos(p, 50, 700)
        PDF_show(p, "Hello, world/signed!")
        PDF_continue_text(p, "(says C)")
        PDF_end_page_ext(p, "")

        PDF_end_document(p, "")

        # fetch the buffer containing the PDF document
        contents = PDF_get_buffer(p)

    except PDFlibException:
        print_exc(file=stdout)

except Exception:
    print_exc(file=stdout)

# ------------------ Step 2: sign the document -----------------
    
# create a new PLOP object
plop = PLOP_new()

try:
    try:
        optlist = "searchpath={%s}" % searchpath
        PLOP_set_option(plop, optlist)

        # Create a virtual PVF file from the data in memory
        PLOP_create_pvf(plop, "/pvf/hello.pdf", contents, "")

        # open unsigned input document from PVF file
        doc = PLOP_open_document(plop, "/pvf/hello.pdf", "")
        if (doc == -1):
            raise PLOPException("Error: %s" % PLOP_get_errmsg(plop))

        # sign the output file
        if (PLOP_create_file(plop, out_filename, sign_opts) == -1):
            raise PLOPException("Error: %s" % PLOP_get_errmsg(plop))

        # close the output file
        PLOP_close_document(plop, doc)


        # delete PVF file containing input document
        if (PLOP_delete_pvf(plop, "/pvf/hello.pdf") == -1):
            raise PLOPException("Error: %s" % PLOP_get_errmsg(plop))

    except PLOPException:
        print_exc(file=stdout)

except Exception:
    print_exc(file=stdout)

finally:
    PLOP_delete(plop)
    PDF_delete(p)
